#ifndef KBF_EXCEPTION_H
#define KBF_EXCEPTION_H

#include <exception>
#include <string>
#include <utility>

#include <esp_err.h>

namespace kbf::exception {
    class KBFError : public std::exception {
    public:
        explicit KBFError(const std::string &message) : message("KBF error: " + message) {}

        const std::string message;

        [[nodiscard]] const char *what() const _GLIBCXX_TXN_SAFE_DYN _GLIBCXX_USE_NOEXCEPT override {
            return message.c_str();
        }
    };

    class ESPError : public std::exception {
        const esp_err_t err;
    public:
        explicit ESPError(const esp_err_t err) : err(err) {};

        [[nodiscard]] const char *what() const _GLIBCXX_TXN_SAFE_DYN _GLIBCXX_USE_NOEXCEPT override {
            return esp_err_to_name(err);
        }
    };

    class RTOSError : public std::exception {
        const std::string message;
    public:
        explicit RTOSError(const uint32_t err) : message("RTOS error: " + std::to_string(err)) {};

        [[nodiscard]] const char *what() const _GLIBCXX_TXN_SAFE_DYN _GLIBCXX_USE_NOEXCEPT override {
            return message.c_str();
        }
    };

    class OutOfMemoryError : public std::exception {
    };

    class InvalidArgument : public std::exception {
        const char *msg;
    public:
        explicit InvalidArgument(const char *msg) : msg(msg) {}

        [[nodiscard]] const char *what() const _GLIBCXX_TXN_SAFE_DYN _GLIBCXX_USE_NOEXCEPT override {
            return msg;
        }
    };
}

#endif //KBF_EXCEPTION_H
