#ifndef KBF_WIFI_H
#define KBF_WIFI_H

#include <string>
#include <vector>

#include "net.h"

namespace kbf::wifi {
    void stop();

    enum class Mode {
        OFF,
        STA,
        AP,
        DUAL
    };

    Mode mode();

    class STA {
    public:
        STA() : aid(0), mac{} {};

        explicit STA(wifi_event_ap_staconnected_t &data) : aid(data.aid), mac(data.mac) {};

        const int           aid;
        const kbf::net::MAC mac;
    };

    namespace ap {
        void start(
                std::string ssid,
                std::string password,
                net::IP ip = net::IP("192.168.4.1"),
                net::IP netmask = net::IP("255.255.255.0")
        );

        void stop();

        [[nodiscard]] const std::string &ssid();

        [[nodiscard]] const std::string &password();

        extern void (*onConnect)(STA &);

        extern void (*onDisconnect)(STA &);
    }

    struct APInfo {
        std::string ssid;
        int8_t rssi;
    };

    namespace sta {
        void start();

        void stop();

        bool connect(std::string ssid, std::string password, bool async = false, int maxAttempts = 3);

        void disconnect();

        [[nodiscard]] bool connected();

        [[nodiscard]] const std::string &ssid();

        [[nodiscard]] const std::string &password();

        [[nodiscard]] const net::IP &ip();

        extern void (*onConnect)();

        extern void (*onIp)();

        extern void (*onDisconnect)();

        void startScan(void *data);

        extern void (*onScanDone)(std::vector<APInfo> &apList, void *data);
    }

    namespace dual {
        void start(
                std::string ssid,
                std::string password,
                net::IP ip = net::IP("192.168.4.1"),
                net::IP netmask = net::IP("255.255.255.0")
        );
    }

    namespace exception {
        class InvalidMode : public std::exception {
        };
    }
}

#endif //KBF_WIFI_H
