#include "kbf/wifi.h"

#include <esp_log.h>
#include <esp_err.h>
#include <esp_wifi.h>

#define KBF_INTERNAL

#include "kbf/internal/wifi.h"
#include "kbf/nvs.h"
#include "kbf/macros.h"

using namespace kbf;
using namespace kbf::wifi;
using std::string;

bool internal::initialized = false;

Mode internal::mode = Mode::OFF;

static constexpr const char *const TAG = "kbf::wifi";

void internal::init() {
    ESP_LOGI(TAG, "internal::%s()", __func__);

    if (initialized) {
        ESP_LOGI(TAG, "already initialized");
        return;
    }

    CHECK(esp_netif_init());
    CHECK(esp_event_loop_create_default());

    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    auto err = esp_wifi_init(&cfg);
    if (err == ESP_ERR_NVS_NOT_INITIALIZED) {
        kbf::nvs::init();
        CHECK(esp_wifi_init(&cfg));
    } else {
        CHECK(err);
    }

    initialized = true;
}

void internal::setMode(Mode m) {
    ESP_LOGI(TAG, "internal::%s(%d)", __func__, (int) m);
    internal::mode = m;
}

Mode wifi::mode() { return internal::mode; }

void dual::start(string ssid, std::string password, net::IP ip, net::IP netmask) {
    ESP_LOGI(TAG, "%s()", __func__);

    sta::start();
    ap::start(std::move(ssid), std::move(password), ip, netmask);
}

void wifi::stop() {
    ESP_LOGI(TAG, "%s()", __func__);

    // TODO stop all at once
    if (internal::mode == Mode::STA || internal::mode == Mode::DUAL) {
        sta::stop();
    }
    if (internal::mode == Mode::AP) {
        ap::stop();
    }
}
